package lib.ui;

import kernel.Log;
import util.Color;
import util.Vec.Vec2;
import kernel.ui.WindowContext;

class ReactiveUI {
	private final context:WindowContext;
	private final children:Array<IElement>;

	public function new(context:WindowContext, children:Array<IElement>) {
		this.context = context;
		this.children = children;

		for (child in children) {
			child.changed.handle(v -> {
				render();
			});
		}
	}

	public function render() {
		var size = context.getSize();

		var screen = renderChildren(children, size);

		writeToContext(screen);
	}

	private function writeToContext(screen:Canvas) {
		var currentBg:Color = Black;
		var currentFg:Color = White;

		var currentLine = 0;

		context.setBackgroundColor(currentBg);
		context.setTextColor(currentFg);
		context.setCursorPos(0, 0);

		for (key => pixel in screen) {
			if (key.y != currentLine) {
				currentLine = key.y;
				context.setCursorPos(key.x, key.y);
			}

			if (pixel == null) {
				context.write(' ');
			} else {
				if (pixel.bg != currentBg) {
					context.setBackgroundColor(pixel.bg);
					currentBg = pixel.bg;
				}

				if (pixel.textColor != currentFg) {
					context.setTextColor(pixel.textColor);
					currentFg = pixel.textColor;
				}

				context.write(pixel.char);
			}
		}
	}

	public static function renderChildren(children:Array<IElement>, bounds:Vec2<Int>):Canvas {
		var rtn:Canvas = new Canvas();

		var writePoint:Vec2<Int> = {x: 0, y: 0};

		for (child in children) {
			if (bounds.y - writePoint.y <= 0) {
				// No more space to render children
				Log.debug("No more space");
				break;
			}

			var childRender = child.render({
				x: bounds.x,
				y: bounds.y - writePoint.y
			});

			rtn.combine(childRender, writePoint);

			writePoint = {x: 0, y: writePoint.y + childRender.hight()};
		}

		return rtn;
	}
}
